package component

import androidx.compose.animation.AnimatedContent
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.Clear
import androidx.compose.material.icons.filled.MoreVert
import androidx.compose.material.icons.filled.Search
import androidx.compose.material.icons.outlined.Info
import androidx.compose.material.icons.outlined.Place
import androidx.compose.material.icons.outlined.Settings
import androidx.compose.material3.*
import androidx.compose.runtime.*
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.semantics.isTraversalGroup
import androidx.compose.ui.semantics.semantics
import androidx.compose.ui.semantics.traversalIndex
import androidx.compose.ui.unit.dp
import cafe.adriel.voyager.navigator.LocalNavigator
import cafe.adriel.voyager.navigator.currentOrThrow
import data.ENumber
import tab.Licenses

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun Search(eNumbers: List<ENumber>) {
    var text by rememberSaveable { mutableStateOf("") }
    var expanded by rememberSaveable { mutableStateOf(false) }
    val menuExpanded = remember { mutableStateOf(false) }

    Column (
        Modifier
            .fillMaxWidth()
            .padding(if (!expanded) 8.dp else 0.dp)
            .semantics { isTraversalGroup = true }
    ) {
        SearchBar(
            modifier = Modifier
                .align(Alignment.CenterHorizontally)
                .semantics { traversalIndex = 0f },
            query = text,
            onQueryChange = { text = it },
            onSearch = { expanded = false },
            onActiveChange = { expanded = it },
            active = expanded,
            placeholder = { Text("Search E Numbers") },
            leadingIcon = {
                IconButton(
                    enabled = expanded,
                    onClick = { if (expanded) expanded = false }
                ) {
                    AnimatedContent(
                        targetState = expanded,
                    ) { targetExpanded ->
                        if (targetExpanded) {
                            Icon(Icons.AutoMirrored.Filled.ArrowBack, "Exit Search")
                        } else {
                            Icon(Icons.Filled.Search, "Navigation Menu")
                        }
                    }
                }
            },
            trailingIcon = {
                Row {
                    if (text.isNotEmpty()) {
                        IconButton(onClick = { text = "" }) {
                            Icon(imageVector = Icons.Filled.Clear, contentDescription = "Clear Search")
                        }
                    }
                    if (!expanded) {
                        IconButton(onClick = { menuExpanded.value = !menuExpanded.value }) {
                            Icon(imageVector = Icons.Filled.MoreVert, contentDescription = "Dropdown Menu")
                        }
                        OptionsMenu(menuExpanded)
                    }
                }
            }
        ) {
            Column(Modifier.verticalScroll(rememberScrollState())) {
                searchSuggestion(query = text, eNumbers = eNumbers).forEach {
                    ListItem(
                        headlineContent = { Text(text = it.number) },
                        modifier = Modifier
                            .clickable {
                                text = it.number
                                expanded = false
                            }
                            .fillMaxWidth()
                            .padding(horizontal = 16.dp, vertical = 4.dp)
                    )
                }
            }
        }
    }
}

@Composable
private fun OptionsMenu(expanded: MutableState<Boolean>) {
    val navigator = LocalNavigator.currentOrThrow

    DropdownMenu(
        expanded = expanded.value,
        onDismissRequest = { expanded.value = false },
    ) {
        DropdownMenuItem(
            text = { Text("Settings") },
            onClick = { /* TODO open settings screen */ },
            leadingIcon = { Icon(imageVector = Icons.Outlined.Settings, contentDescription = "Open settings") }
        )
        DropdownMenuItem(
            text = { Text("About") },
            onClick = { /* TODO open about screen */ },
            leadingIcon = { Icon(imageVector = Icons.Outlined.Place, contentDescription = "Information about the application") }
        )
        DropdownMenuItem(
            text = { Text("Licenses") },
            onClick = { navigator.push(Licenses) },
            leadingIcon = { Icon(imageVector = Icons.Outlined.Info, contentDescription = "Information about Licenses") }
        )
    }
}

private fun searchSuggestion(query: String, eNumbers: List<ENumber>): List<ENumber> {
    if (query.isEmpty()) {
        return eNumbers
    }

    val filtered = eNumbers.filter { eNumber ->
        eNumber.number.contains(query, ignoreCase = true)
    }

    return filtered
}
